function [ out ] = BGDeconv ( C, B )
% BGConv deconvolves C to A, with B being a known convolving matrix
% e.g. C is the reduced photon statistics, B is the reduced background
% photon statistics
%      ---> A is the photon statistics without the background

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% INPUT
% C - measured reduced photon statistics matrix
% B - reduced background photon statistics matrix

% OUTPUT
% A - the reduced photon statistics of the source w/o background

% --- ERRORS --- 
% - C or B not square
if or(size(C,1)~=size(C,2),size(B,1)~=size(B,2))
    error('Matrices must be square');
end
% - size C =/= size B
if size(C,1)~=size(B,1)
    error('Matrices must be the same size');
end
% --------------

S = size(C,2); % Finding the size of the problem

out = zeros(S,S);
A = zeros(S,S);

% Compute the deconvolution using the convolution theorem
A = abs(ifft2(fft2(C)./fft2(B)));

out = A;