function [ B ] = Bground( mean, maxnum )
% BGROUND - Creates a Poissonian background matrix which can be used to
% transform an ideal state into a non-ideal state. Whereas the Loss
% matrix removes photons from the field, Bground matrix adds them. However,
% the form with which they do this is different.

% mean - mean of the desired Poissonian distribution
% maxnum - maximum number of photons to consider

% To ensure the matrix is physical, maxnum must be chosen to be
% sufficiently high that the Poissonian distribution is not significantly
% truncated.

N = maxnum;

B = zeros(N+1,N+1);

numvals = (0:N)';
poiss = poisspdf(numvals, mean);

for m = 1:(N+1);
    B(m:(N+1), m) = poiss(1:((N+1)-(m-1)));
end
    

