%NEWTONANALYSIS 
% Takes clicks statistics and finds optimum values of l1,l2,b1,b2 (the loss
% parameters and background parameters) by using Newton's method.

% An initial guess is provided (e.g. losses ~0.9, background ~0.1), and
% then Newton canes it out by calculating local derivatives and deciding a
% step direction, d, in which to go to find the next point.

% The parameters [l1,l2,b1,b2] will sometimes be referred to as
% [x1,x2,x3,x4]

% Max number of photons to consider i.e. size of state
N = 8;

% Distances in parameter space for calculating derivatives
% h = [h(l1), h(l2), h(b1), h(b2)]
h = [0.01, 0.01, 0.01, 0.01]';
gamma = [1,1,1,1]';

Imax = 50;
iter = 1;

d = zeros(4,Imax);

dump = zeros(N+1,1); % The minimised states are irrelevant

% Starting vector
x_0 = [0.76,0.96,0.15,0.3]';

x = zeros(5, Imax);
x(1:4,1) = x_0;

T = zeros(5,21);
D = zeros(14);

G = zeros(4,1);
H = zeros(4,4);

% Find statistics from click counts
P = click/sum(sum(click));
M = zeros(81,N+1);

% Use all P elements to make p an 81x1 vector
prow = 1;
for n = 1:9
    for m = 1:9
        p(prow,1) = P(n,m);
        prow = prow + 1;
    end
end

for iter = 1:Imax
    
    % ---- Calculating the function points for derivatives ----

    % Temporary 5x21 matrix, T, stores these points and their residuals
    %           - T(5,1) = 1 function value at X(1:4,iter)
    %           - T(5,2:9) = 8 function values for single variable
    T(1:4,1) = x(1:4,iter); % Load point

    T(1:4,2) = x(1:4,iter) + [h(1),0,0,0]'; % Load single variable points
    T(1:4,3) = x(1:4,iter) - [h(1),0,0,0]';
    T(1:4,4) = x(1:4,iter) + [0,h(2),0,0]';
    T(1:4,5) = x(1:4,iter) - [0,h(2),0,0]';
    T(1:4,6) = x(1:4,iter) + [0,0,h(3),0]';
    T(1:4,7) = x(1:4,iter) - [0,0,h(3),0]';
    T(1:4,8) = x(1:4,iter) + [0,0,0,h(4)]';
    T(1:4,9) = x(1:4,iter) - [0,0,0,h(4)]';


    for u=1:9
        % Calculate loss matrices
        L1 = Loss(T(1,u),N); L2 = Loss(T(2,u),N);

        % Calculate background matrices
        B1 = Bground(T(3,u),N); B2 = Bground(T(4,u),N);

        % Calculate working matrices - initially need P = Q*rho*R
        Q = P_kn1*L1*B1;
        R = B2'*L2'*P_kn2';

        % Create 81x(N+1) M matrix - to convert to form p = Mx
        mrow = 1;
        for n = 1:9
            for m = 1:9
            M(mrow,:) = Q(n,:).*R(:,m)';    
            mrow = mrow + 1;
            end
        end

        % Do the least squares minimisation (i.e. compute the value of the
        % function!)
        [dump, T(5,u)] = lsqnonneg(M,p);
        if u==1
            x(5,iter) = T(5,u);
        end
    
    end


     % ---- Calculating the derivative values ----
    % A 14-vector, D, for 
    %           - D(1:4) = 1st derivatives (1,2,3,4)

    % 1st derivatives
    D(1) = (T(5,2) - T(5,3))/(2*h(1));
    D(2) = (T(5,4) - T(5,5))/(2*h(2));
    D(3) = (T(5,6) - T(5,7))/(2*h(3));
    D(4) = (T(5,8) - T(5,9))/(2*h(4));


     % ---- Constructing the Gradient Vector ----
     G = zeros(4,1);
     G = [D(1),D(2),D(3),D(4)]';
     

     % ---- Moving to a new point in parameter space ----
     % Points are recorded in 5 x Imax matrix, X, where Imax = max number of iterations
     d(1:4,iter) = G;
     x(1:4,iter+1) = x(1:4,iter) - gamma.*d(1:4,iter);
    
     iter
 end
 
 
 